function [H_PHI_out,H_PSI_out] = green_coords_3d_urago3_hessian_vectorized(X,T,zt_in)

nv = length(X);
nf = length(T);

x1 = X(T(:,1),:); x2 = X(T(:,2),:); x3 = X(T(:,3),:);
N = cross(x2-x1,x3-x1);
nn = normv(N);
N = N./repmat(nn,1,3);

dx1 = x2 - x3; 
dx2 = x3 - x1;
dx3 = x1 - x2;

cNdx1 = cross(N,dx1,2);
cNdx2 = cross(N,dx2,2);
cNdx3 = cross(N,dx3,2);

l1a = normv(dx1); 
l2a = normv(dx2); 
l3a = normv(dx3); 

chunk_size = 500;
curr = 0;
while curr <= size(zt_in,1)
    tic
    if curr+chunk_size <= size(zt_in,1)
        zt = zt_in(curr+1:curr+chunk_size,:);
    else
        zt = zt_in(curr+1:end,:);
    end
        
    m = size(zt,1);

    [I,J] = meshgrid(1:m,1:nf);
    e1x = reshape(x1(J,1)-zt(I,1),nf,m)'; e1y = reshape(x1(J,2)-zt(I,2),nf,m)'; e1z = reshape(x1(J,3)-zt(I,3),nf,m)';
    e2x = reshape(x2(J,1)-zt(I,1),nf,m)'; e2y = reshape(x2(J,2)-zt(I,2),nf,m)'; e2z = reshape(x2(J,3)-zt(I,3),nf,m)';
    e3x = reshape(x3(J,1)-zt(I,1),nf,m)'; e3y = reshape(x3(J,2)-zt(I,2),nf,m)'; e3z = reshape(x3(J,3)-zt(I,3),nf,m)';

    gx = (e1x + e2x + e3x)/3;
    gy = (e1y + e2y + e3y)/3;
    gz = (e1z + e2z + e3z)/3;

    l1 = repmat(l1a',m,1);
    l2 = repmat(l2a',m,1);
    l3 = repmat(l3a',m,1);

    [J1x,J1y,J1z] = cross_matrices(e3x, e3y, e3z, e2x, e2y, e2z);
    [J2x,J2y,J2z] = cross_matrices(e1x, e1y, e1z, e3x, e3y, e3z);
    [J3x,J3y,J3z] = cross_matrices(e2x, e2y, e2z, e1x, e1y, e1z);

    R1 = sqrt(e1x.^2 + e1y.^2 + e1z.^2);
    R2 = sqrt(e2x.^2 + e2y.^2 + e2z.^2);
    R3 = sqrt(e3x.^2 + e3y.^2 + e3z.^2);

    Nx = repmat(N(:,1)',m,1);
    Ny = repmat(N(:,2)',m,1);
    Nz = repmat(N(:,3)',m,1);

    sR1 = R2 + R3;
    sR2 = R3 + R1;
    sR3 = R1 + R2;

    sc1 = 1./l1.*log((sR1 + l1)./(sR1 - l1));
    sc2 = 1./l2.*log((sR2 + l2)./(sR2 - l2));
    sc3 = 1./l3.*log((sR3 + l3)./(sR3 - l3));

    Lx = J1x.*sc1 + J2x.*sc2 + J3x.*sc3;
    Ly = J1y.*sc1 + J2y.*sc2 + J3y.*sc3;
    Lz = J1z.*sc1 + J2z.*sc2 + J3z.*sc3;

    Mx = repmat(dx1(:,1)',m,1).*sc1 + repmat(dx2(:,1)',m,1).*sc2 + repmat(dx3(:,1)',m,1).*sc3;
    My = repmat(dx1(:,2)',m,1).*sc1 + repmat(dx2(:,2)',m,1).*sc2 + repmat(dx3(:,2)',m,1).*sc3;
    Mz = repmat(dx1(:,3)',m,1).*sc1 + repmat(dx2(:,3)',m,1).*sc2 + repmat(dx3(:,3)',m,1).*sc3;

    omega = -calc_omega_vectorized(e1x,e2x,e3x, e1y,e2y,e3y, e1z,e2z,e3z);    

    [Px,Py,Pz] = cross_matrices(Nx,Ny,Nz,Mx,My,Mz);
    Px = (Px + omega.*Nx)/(4*pi);
    Py = (Py + omega.*Ny)/(4*pi);
    Pz = (Pz + omega.*Nz)/(4*pi);

    Jp1 = dot_matrices(J1x,J1y,J1z,Px,Py,Pz)./repmat(nn',m,1);
    Jp2 = dot_matrices(J2x,J2y,J2z,Px,Py,Pz)./repmat(nn',m,1);
    Jp3 = dot_matrices(J3x,J3y,J3z,Px,Py,Pz)./repmat(nn',m,1);

    eR1x = e1x./R1; eR1y = e1y./R1; eR1z = e1z./R1;
    eR2x = e2x./R2; eR2y = e2y./R2; eR2z = e2z./R2;
    eR3x = e3x./R3; eR3y = e3y./R3; eR3z = e3z./R3;

    dsc1 = 2./((sR1 + l1).*(sR1 - l1));
    dsc2 = 2./((sR2 + l2).*(sR2 - l2));
    dsc3 = 2./((sR3 + l3).*(sR3 - l3));
    
    E1x = (eR2x + eR3x).*dsc1; E1y = (eR2y + eR3y).*dsc1; E1z = (eR2z + eR3z).*dsc1; 
    E2x = (eR3x + eR1x).*dsc2; E2y = (eR3y + eR1y).*dsc2; E2z = (eR3z + eR1z).*dsc2; 
    E3x = (eR1x + eR2x).*dsc3; E3y = (eR1y + eR2y).*dsc3; E3z = (eR1z + eR2z).*dsc3; 
    Et.x1 = E1x; Et.y1 = E2x; Et.z1 = E3x;
    Et.x2 = E1y; Et.y2 = E2y; Et.z2 = E3y;    
    Et.x3 = E1z; Et.y3 = E2z; Et.z3 = E3z;
    
    oR1 = 1./R2 + 1./R3;
    oR2 = 1./R3 + 1./R1;
    oR3 = 1./R1 + 1./R2;

    dscor1 = dsc1.*oR1;
    dscor2 = dsc2.*oR2;
    dscor3 = dsc3.*oR3;
        
    grad_omega_x = dot_matrices(J1x,J2x,J3x,dscor1,dscor2,dscor3);
    grad_omega_y = dot_matrices(J1y,J2y,J3y,dscor1,dscor2,dscor3);
    grad_omega_z = dot_matrices(J1z,J2z,J3z,dscor1,dscor2,dscor3);
    
    dxnct.x1 = repmat(cNdx1(:,1)',m,1); dxnct.y1 = repmat(cNdx1(:,2)',m,1); dxnct.z1 = repmat(cNdx1(:,3)',m,1);
    dxnct.x2 = repmat(cNdx2(:,1)',m,1); dxnct.y2 = repmat(cNdx2(:,2)',m,1); dxnct.z2 = repmat(cNdx2(:,3)',m,1);
    dxnct.x3 = repmat(cNdx3(:,1)',m,1); dxnct.y3 = repmat(cNdx3(:,2)',m,1); dxnct.z3 = repmat(cNdx3(:,3)',m,1);
    
    J_MNct = multiply_matrices(Et, dxnct);
    
    J_P.x1 = (J_MNct.x1 + grad_omega_x.*Nx)/(4*pi);
    J_P.x2 = (J_MNct.x2 + grad_omega_y.*Nx)/(4*pi);
    J_P.x3 = (J_MNct.x3 + grad_omega_z.*Nx)/(4*pi);
    J_P.y1 = (J_MNct.y1 + grad_omega_x.*Ny)/(4*pi);
    J_P.y2 = (J_MNct.y2 + grad_omega_y.*Ny)/(4*pi);
    J_P.y3 = (J_MNct.y3 + grad_omega_z.*Ny)/(4*pi);
    J_P.z1 = (J_MNct.z1 + grad_omega_x.*Nz)/(4*pi);
    J_P.z2 = (J_MNct.z2 + grad_omega_y.*Nz)/(4*pi);
    J_P.z3 = (J_MNct.z3 + grad_omega_z.*Nz)/(4*pi);

    H_PSI.xx = -J_P.x1;
    H_PSI.xy = -J_P.y1;
    H_PSI.xz = -J_P.z1;
    H_PSI.yy = -J_P.y2;
    H_PSI.yz = -J_P.z2;
    
    H_PSI_all = [H_PSI.xx,H_PSI.xy,H_PSI.xz,H_PSI.yy,H_PSI.yz];
    H_PSI_all = reshape(H_PSI_all',nf,5*m)';

    H_PSI_out(curr*5 + 1: curr*5 + size(H_PSI_all,1),:) = H_PSI_all;

    dx1a.x = repmat(dx1(:,1)',m,1);
    dx1a.y = repmat(dx1(:,2)',m,1);
    dx1a.z = repmat(dx1(:,3)',m,1);

    dx2a.x = repmat(dx2(:,1)',m,1);
    dx2a.y = repmat(dx2(:,2)',m,1);
    dx2a.z = repmat(dx2(:,3)',m,1);
    
    dx3a.x = repmat(dx3(:,1)',m,1);
    dx3a.y = repmat(dx3(:,2)',m,1);
    dx3a.z = repmat(dx3(:,3)',m,1);
    
    [H_PHI_1.xx, dum,dumy]         = cross_matrices(dx1a.x, dx1a.y, dx1a.z, J_P.x1, J_P.x2, J_P.x3);
    [H_PHI_1.xy, H_PHI_1.yy, dumy] = cross_matrices(dx1a.x, dx1a.y, dx1a.z, J_P.y1, J_P.y2, J_P.y3);
    [H_PHI_1.xz, H_PHI_1.yz, dumy] = cross_matrices(dx1a.x, dx1a.y, dx1a.z, J_P.z1, J_P.z2, J_P.z3);
    
    [H_PHI_2.xx, dum,dumy]         = cross_matrices(dx2a.x, dx2a.y, dx2a.z, J_P.x1, J_P.x2, J_P.x3);
    [H_PHI_2.xy, H_PHI_2.yy, dumy] = cross_matrices(dx2a.x, dx2a.y, dx2a.z, J_P.y1, J_P.y2, J_P.y3);
    [H_PHI_2.xz, H_PHI_2.yz, dumy] = cross_matrices(dx2a.x, dx2a.y, dx2a.z, J_P.z1, J_P.z2, J_P.z3);

    [H_PHI_3.xx, dum,dumy]         = cross_matrices(dx3a.x, dx3a.y, dx3a.z, J_P.x1, J_P.x2, J_P.x3);
    [H_PHI_3.xy, H_PHI_3.yy, dumy] = cross_matrices(dx3a.x, dx3a.y, dx3a.z, J_P.y1, J_P.y2, J_P.y3);
    [H_PHI_3.xz, H_PHI_3.yz, dumy] = cross_matrices(dx3a.x, dx3a.y, dx3a.z, J_P.z1, J_P.z2, J_P.z3);
    
    H_PHI_all_1 = -[H_PHI_1.xx,H_PHI_1.xy,H_PHI_1.xz,H_PHI_1.yy,H_PHI_1.yz]./repmat(nn',m,5);
    H_PHI_all_1 = reshape(H_PHI_all_1',nf,5*m)';

    H_PHI_all_2 = -[H_PHI_2.xx,H_PHI_2.xy,H_PHI_2.xz,H_PHI_2.yy,H_PHI_2.yz]./repmat(nn',m,5);
    H_PHI_all_2 = reshape(H_PHI_all_2',nf,5*m)';
    
    H_PHI_all_3 = -[H_PHI_3.xx,H_PHI_3.xy,H_PHI_3.xz,H_PHI_3.yy,H_PHI_3.yz]./repmat(nn',m,5);
    H_PHI_all_3 = reshape(H_PHI_all_3',nf,5*m)';

    f2v1 = sparse(1:nf,T(:,1),ones(nf,1),nf,nv);
    f2v2 = sparse(1:nf,T(:,2),ones(nf,1),nf,nv);
    f2v3 = sparse(1:nf,T(:,3),ones(nf,1),nf,nv);

    H_PHI_all = zeros(5*m,nv);
    H_PHI_all = H_PHI_all + H_PHI_all_1*f2v1;
    H_PHI_all = H_PHI_all + H_PHI_all_2*f2v2;
    H_PHI_all = H_PHI_all + H_PHI_all_3*f2v3;    

    H_PHI_out(curr*5 + 1: curr*5 + size(H_PHI_all,1),:) = H_PHI_all;
     
    curr = curr+chunk_size;
    toc
end